@@ -34,6 +34,10 @@
 
 namespace Breeze
 {
+
+    //* contrast for arrow and treeline rendering
+    static const qreal arrowShade = 0.15;
+
     //____________________________________________________________________
     Helper::Helper( KSharedConfig::Ptr config ):
         _config( config )
@@ -102,6 +106,21 @@ namespace Breeze
 
     }
 
+    //____________________________________________________________________
+    QColor Helper::focusOutlineColor( const QPalette& palette ) const
+    { return KColorUtils::mix( focusColor( palette ), palette.color( QPalette::WindowText ), 0.15 ); }
+
+    //____________________________________________________________________
+    QColor Helper::hoverOutlineColor( const QPalette& palette ) const
+    { return KColorUtils::mix( hoverColor( palette ), palette.color( QPalette::WindowText ), 0.15 ); }
+
+    //____________________________________________________________________
+    QColor Helper::buttonFocusOutlineColor( const QPalette& palette ) const
+    { return KColorUtils::mix( focusColor( palette ), palette.color( QPalette::ButtonText ), 0.15 ); }
+
+    //____________________________________________________________________
+    QColor Helper::buttonHoverOutlineColor( const QPalette& palette ) const
+    { return KColorUtils::mix( hoverColor( palette ), palette.color( QPalette::ButtonText ), 0.15 ); }
 
     //____________________________________________________________________
     QColor Helper::sidePanelOutlineColor( const QPalette& palette, bool hasFocus, qreal opacity, AnimationMode mode ) const
@@ -131,7 +150,6 @@ namespace Breeze
 
     //____________________________________________________________________
     QColor Helper::sidePanelBackgroundColor( const QPalette& palette, QPalette::ColorGroup role ) const
-    // { return KColorUtils::mix( palette.color( role, QPalette::Window ), palette.color( role, QPalette::Base ), 0.6 ); }
     { return palette.color( role, QPalette::Window ); }
 
     //____________________________________________________________________
@@ -165,11 +183,24 @@ namespace Breeze
         return copy;
     }
 
+    //____________________________________________________________________
+    QColor Helper::arrowColor( const QPalette& palette, QPalette::ColorGroup group, QPalette::ColorRole role ) const
+    {
+        switch( role )
+        {
+            case QPalette::Text: return KColorUtils::mix( palette.color( group, QPalette::Text ), palette.color( group, QPalette::Base ), arrowShade );
+            case QPalette::WindowText: return KColorUtils::mix( palette.color( group, QPalette::WindowText ), palette.color( group, QPalette::Window ), arrowShade );
+            case QPalette::ButtonText: return KColorUtils::mix( palette.color( group, QPalette::ButtonText ), palette.color( group, QPalette::Button ), arrowShade );
+            default: return palette.color( group, role );
+        }
+
+    }
+
     //____________________________________________________________________
     QColor Helper::arrowColor( const QPalette& palette, bool mouseOver, bool hasFocus, qreal opacity, AnimationMode mode ) const
     {
 
-        QColor outline( palette.color( QPalette::WindowText ) );
+        QColor outline( arrowColor( palette, QPalette::WindowText ) );
         if( mode == AnimationHover )
         {
 
@@ -205,23 +236,32 @@ namespace Breeze
         if( mode == AnimationHover )
         {
 
-            const QColor focus( focusColor( palette ) );
-            const QColor hover( hoverColor( palette ) );
-            if( hasFocus ) outline = KColorUtils::mix( focus, hover, opacity );
-            else outline = KColorUtils::mix( outline, hover, opacity );
+            if( hasFocus )
+            {
+                const QColor focus( buttonFocusOutlineColor( palette ) );
+                const QColor hover( buttonHoverOutlineColor( palette ) );
+                outline = KColorUtils::mix( focus, hover, opacity );
+
+            } else {
+
+                const QColor hover( hoverColor( palette ) );
+                outline = KColorUtils::mix( outline, hover, opacity );
+
+            }
 
         } else if( mouseOver ) {
 
-            outline = hoverColor( palette );
+            if( hasFocus ) outline = buttonHoverOutlineColor( palette );
+            else outline = hoverColor( palette );
 
         } else if( mode == AnimationFocus ) {
 
-            const QColor focus( focusColor( palette ) );
+            const QColor focus( buttonFocusOutlineColor( palette ) );
             outline = KColorUtils::mix( outline, focus, opacity );
 
         } else if( hasFocus ) {
 
-            outline = focusColor( palette );
+            outline = buttonFocusOutlineColor( palette );
 
         }
 
@@ -230,10 +270,13 @@ namespace Breeze
     }
 
     //____________________________________________________________________
-    QColor Helper::buttonBackgroundColor( const QPalette& palette, bool mouseOver, bool hasFocus, qreal opacity, AnimationMode mode ) const
+    QColor Helper::buttonBackgroundColor( const QPalette& palette, bool mouseOver, bool hasFocus, bool sunken, qreal opacity, AnimationMode mode ) const
     {
 
-        QColor background( palette.color( QPalette::Button ) );
+        QColor background( sunken ?
+            KColorUtils::mix( palette.color( QPalette::Button ), palette.color( QPalette::ButtonText ), 0.2 ):
+            palette.color( QPalette::Button ) );
+
         if( mode == AnimationHover )
         {
 
@@ -432,15 +475,41 @@ namespace Breeze
     }
 
     //______________________________________________________________________________
-    void Helper::renderFocusRect( QPainter* painter, const QRect& rect, const QColor& color ) const
+    void Helper::renderFocusRect( QPainter* painter, const QRect& rect, const QColor& color, const QColor& outline, Sides sides ) const
     {
         if( !color.isValid() ) return;
 
         painter->save();
+        painter->setRenderHints( QPainter::Antialiasing );
         painter->setBrush( color );
-        painter->setPen( Qt::NoPen );
-        painter->drawRect( rect );
+
+        if( !( outline.isValid() && sides ) )
+        {
+
+            painter->setPen( Qt::NoPen );
+            painter->drawRect( rect );
+
+        } else {
+
+            painter->setClipRect( rect );
+
+            QRectF copy( rect );
+            copy.adjust( 0.5, 0.5, -0.5, -0.5 );
+
+            const qreal radius( frameRadius( -1.0 ) );
+            if( !(sides&SideTop) ) copy.adjust( 0, -radius, 0, 0 );
+            if( !(sides&SideBottom) ) copy.adjust( 0, 0, 0, radius );
+            if( !(sides&SideLeft) ) copy.adjust( -radius, 0, 0, 0 );
+            if( !(sides&SideRight) ) copy.adjust( 0, 0, radius, 0 );
+
+            painter->setPen( outline );
+            // painter->setBrush( Qt::NoBrush );
+            painter->drawRoundedRect( copy, radius, radius );
+
+        }
+
         painter->restore();
+        return;
     }
 
     //______________________________________________________________________________
@@ -475,7 +544,7 @@ namespace Breeze
 
             painter->setPen( outline );
             frameRect.adjust( 0.5, 0.5, -0.5, -0.5 );
-            radius -= 1.0;
+            radius = qMax( radius - 1, qreal( 0.0 ) );
 
         } else {
 
@@ -533,8 +602,7 @@ namespace Breeze
 
             case AllSides:
             {
-                qreal radius( frameRadius() );
-                radius -= 1.0;
+                const qreal radius( frameRadius( -1.0 ) );
                 painter->drawRoundedRect( frameRect, radius, radius );
                 break;
             }
@@ -560,7 +628,7 @@ namespace Breeze
 
             painter->setPen( outline );
             frameRect.adjust( 0.5, 0.5, -0.5, -0.5 );
-            radius -= 1.0;
+            radius = qMax( radius - 1, qreal( 0.0 ) );
 
         } else painter->setPen( Qt::NoPen );
 
@@ -596,18 +664,23 @@ namespace Breeze
 
         } else if( shadow.isValid() ) {
 
+            const qreal shadowRadius = qMax( radius - 1, qreal( 0.0 ) );
             painter->setPen( QPen( shadow, 2 ) );
             painter->setBrush( Qt::NoBrush );
-            painter->drawRoundedRect( shadowRect( frameRect ), radius - 1, radius - 1 );
+            painter->drawRoundedRect( shadowRect( frameRect ), shadowRadius, shadowRadius );
 
         }
 
-        if( outline.isValid() && !hasFocus )
+        if( outline.isValid() )
         {
 
-            painter->setPen( outline );
+            QLinearGradient gradient( frameRect.topLeft(), frameRect.bottomLeft() );
+            gradient.setColorAt( 0, outline.lighter( hasFocus ? 103:101 ) );
+            gradient.setColorAt( 1, outline.darker( hasFocus ? 110:103 ) );
+            painter->setPen( QPen( QBrush( gradient ), 1.0 ) );
+
             frameRect.adjust( 0.5, 0.5, -0.5, -0.5 );
-            radius -= 1.0;
+            radius = qMax( radius - 1, qreal( 0.0 ) );
 
         } else painter->setPen( Qt::NoPen );
 
@@ -654,7 +727,7 @@ namespace Breeze
 
         } else {
 
-            const qreal radius( frameRadius()-0.5 );
+            const qreal radius( frameRadius(-0.5) );
 
             painter->setPen( color );
             painter->setBrush( Qt::NoBrush );
@@ -693,7 +766,7 @@ namespace Breeze
         path.lineTo( ( baseRect.width() - tabWidth )/2, radius );
         path.arcTo( QRectF( QPointF( ( baseRect.width() - tabWidth )/2, 0 ), cornerSize ), 180, -90 );
         path.lineTo( ( baseRect.width() + tabWidth )/2 -1 - radius, 0 );
-        path.arcTo( QRectF(  QPointF( ( baseRect.width() + tabWidth )/2 -1 - 2*radius, 0 ), cornerSize ), 90, -90 );
+        path.arcTo( QRectF(  QPointF( ( baseRect.width() + tabWidth )/2  - 1 - 2*radius, 0 ), cornerSize ), 90, -90 );
         path.lineTo( ( baseRect.width() + tabWidth )/2 -1, baseRect.height()-1 - radius );
         path.arcTo( QRectF( QPointF( ( baseRect.width() + tabWidth )/2 -1, baseRect.height()-1 - 2*radius ), cornerSize ), 180, 90 );
         path.lineTo( baseRect.width()-1, baseRect.height()-1 );
@@ -726,7 +799,7 @@ namespace Breeze
 
             painter->setPen( outline );
             frameRect.adjust( 0.5, 0.5, -0.5, -0.5 );
-            radius -= 1.0;
+            radius = qMax( radius-1, qreal( 0.0 ) );
 
         } else painter->setPen( Qt::NoPen );
 
@@ -782,6 +855,27 @@ namespace Breeze
 
     }
 
+    //______________________________________________________________________________
+    void Helper::renderCheckBoxBackground(
+        QPainter* painter, const QRect& rect,
+        const QColor& color, bool sunken ) const
+    {
+
+        // setup painter
+        painter->setRenderHint( QPainter::Antialiasing, true );
+
+        // copy rect and radius
+        QRectF frameRect( rect );
+        frameRect.adjust( 3, 3, -3, -3 );
+
+        if( sunken ) frameRect.translate(1, 1);
+
+        painter->setPen( Qt::NoPen );
+        painter->setBrush( color );
+        painter->drawRect( frameRect );
+
+    }
+
     //______________________________________________________________________________
     void Helper::renderCheckBox(
         QPainter* painter, const QRect& rect,
@@ -794,8 +888,8 @@ namespace Breeze
 
         // copy rect and radius
         QRectF frameRect( rect );
-        frameRect.adjust( 1, 1, -1, -1 );
-        const qreal radius( frameRadius() );
+        frameRect.adjust( 2, 2, -2, -2 );
+        qreal radius( frameRadius() );
 
         // shadow
         if( sunken )
@@ -819,8 +913,9 @@ namespace Breeze
             painter->setPen( QPen( color, 1 ) );
             painter->setBrush( Qt::NoBrush );
 
+            radius = qMax( radius-1, qreal( 0.0 ) );
             const QRectF contentRect( frameRect.adjusted( 0.5, 0.5, -0.5, -0.5 ) );
-            painter->drawRoundedRect( contentRect, radius - 1.0, radius - 1.0 );
+            painter->drawRoundedRect( contentRect, radius, radius );
 
         }
 
@@ -848,14 +943,14 @@ namespace Breeze
             painter->setRenderHint( QPainter::Antialiasing, false );
 
             QPainterPath path;
-            path.moveTo( 4, 4 );
-            path.lineTo( qreal( Metrics::CheckBox_Size ) -5, 4 );
-            path.lineTo( 4, qreal( Metrics::CheckBox_Size ) - 5 );
-            painter->drawPath( path.translated( rect.topLeft() ) );
+            path.moveTo( markerRect.topLeft() );
+            path.lineTo( markerRect.right() - 1, markerRect.top() );
+            path.lineTo( markerRect.left(), markerRect.bottom()-1 );
+            painter->drawPath( path );
 
         } else if( state == CheckAnimated ) {
 
-            const QRectF markerRect( frameRect.adjusted( 4, 4, -4, -4 ) );
+            const QRectF markerRect( frameRect.adjusted( 3, 3, -3, -3 ) );
             QPainterPath path;
             path.moveTo( markerRect.topRight() );
             path.lineTo( markerRect.center() + animation*( markerRect.topLeft() - markerRect.center() ) );
@@ -871,6 +966,24 @@ namespace Breeze
 
     }
 
+    //______________________________________________________________________________
+    void Helper::renderRadioButtonBackground( QPainter* painter, const QRect& rect, const QColor& color, bool sunken ) const
+    {
+
+        // setup painter
+        painter->setRenderHint( QPainter::Antialiasing, true );
+
+        // copy rect
+        QRectF frameRect( rect );
+        frameRect.adjust( 3, 3, -3, -3 );
+        if( sunken ) frameRect.translate(1, 1);
+
+        painter->setPen( Qt::NoPen );
+        painter->setBrush( color );
+        painter->drawEllipse( frameRect );
+
+    }
+
     //______________________________________________________________________________
     void Helper::renderRadioButton(
         QPainter* painter, const QRect& rect,
@@ -883,7 +996,7 @@ namespace Breeze
 
         // copy rect
         QRectF frameRect( rect );
-        frameRect.adjust( 1, 1, -1, -1 );
+        frameRect.adjust( 2, 2, -2, -2 );
 
         // shadow
         if( sunken )
@@ -1033,7 +1146,6 @@ namespace Breeze
         const QColor& color,
         const QColor& outline,
         const QColor& shadow,
-        bool hasFocus,
         bool sunken ) const
     {
 
@@ -1058,18 +1170,8 @@ namespace Breeze
         if( outline.isValid() )
         {
 
-            if( hasFocus )
-            {
-
-                painter->setPen( QPen( outline, 2 ) );
-                frameRect.adjust( 1, 1, -1, -1 );
-
-            } else {
-
-                painter->setPen( outline );
-                frameRect.adjust( 0.5, 0.5, -0.5, -0.5 );
-
-            }
+            painter->setPen( outline );
+            frameRect.adjust( 0.5, 0.5, -0.5, -0.5 );
 
         } else painter->setPen( Qt::NoPen );
 
@@ -1204,7 +1306,7 @@ namespace Breeze
 
             painter->setPen( outline );
             frameRect.adjust( 0.5, 0.5, -0.5, -0.5 );
-            radius -= 1.0;
+            radius = qMax( radius-1, qreal( 0.0 ) );
 
         } else painter->setPen( Qt::NoPen );
 
